/**
 * Copyright &copy; 2021-2026 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.sys.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.jeeplus.sys.domain.*;
import com.jeeplus.sys.mapper.OfficeMapper;
import com.jeeplus.sys.service.dto.CertDTO;
import com.jeeplus.sys.service.dto.LoginUserOnlineDTO;
import com.jeeplus.sys.service.mapstruct.CertWrapper;
import com.jeeplus.sys.utils.ServletUtils;
import com.jeeplus.sys.utils.ip.AddressUtils;
import com.jeeplus.sys.utils.ip.IpUtils;
import eu.bitwalker.useragentutils.UserAgent;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jeeplus.common.redis.RedisUtils;
import com.jeeplus.common.utils.RequestUtils;
import com.jeeplus.sys.constant.CacheNames;
import com.jeeplus.sys.mapper.UserMapper;
import com.jeeplus.sys.service.dto.OfficeDTO;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.service.mapstruct.UserWrapper;
import com.jeeplus.sys.utils.UserUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 用户管理
 * @author jeeplus
 * @version 2021-09-05
 */
@Service
@Transactional
public class UserService  extends ServiceImpl<UserMapper, User> {

	@Autowired
	private UserWrapper userWrapper;

	@Autowired
	private RedisUtils redisUtils;

	@Autowired
	private SysConfigService sysConfigService;

	@Autowired
	private UserMapper userMapper;

	@Autowired
	private OfficeMapper officeMapper;

	@Autowired
	private CertService certService;

	@Autowired
	private SerialnumTplCService serialnumTplCService;

	@Autowired
	private RosterBaseCService rosterBaseCService;

	/**
	 * 单一登录判断，是否允许登录
	 * @return
	 */
	public boolean isEnableLogin (String loginName) {
		SysConfig sysConfig = sysConfigService.getById ( "1");
		if(sysConfig.getMultiAccountLogin ().equals ( "0" )){ // 不允许一个账号多个客户端登录
			if(sysConfig.getSingleLoginType ().equals ( "1" )){ // 后登陆踢出先登录
				Set <String> keys = RedisUtils.getInstance ( ).keys ( CacheNames.USER_CACHE_TOKEN + loginName+ "*");
				keys.forEach ( key->{
					String token = RedisUtils.getInstance ().get ( key ).toString ();
//					RedisUtils.getInstance ().delete ( CacheNames.USER_CACHE_TOKEN + loginName + ":" + token );
					RedisUtils.getInstance ().set ( CacheNames.USER_CACHE_TOKEN + loginName + ":" + token, "kickOut" );
				} );
			}else{ // 已经登陆禁止再登录
				if(redisUtils.keys (CacheNames.USER_CACHE_TOKEN+ loginName+"*"   ).size () >0){
					return  false;
				};
			}
		}
		return true;
	}


	/**
	 * 获取用户
	 * @param id
	 * @return
	 */
	public UserDTO get(String id) {
		QueryWrapper queryWrapper = new QueryWrapper ();
		queryWrapper.eq ("a.id", id);
		return baseMapper.get (queryWrapper);
	}

	/**
	 * 根据登录名获取用户
	 * @param loginName
	 * @return
	 */
	@Cacheable(cacheNames = CacheNames.USER_CACHE_LOGIN_NAME, key = "#loginName")
	public UserDTO getUserByLoginName(String loginName) {
		QueryWrapper queryWrapper = new QueryWrapper ();
		queryWrapper.eq ("a.login_name", loginName);
		queryWrapper.eq("a.del_flag","0");
		UserDTO userDTO = baseMapper.get(queryWrapper);

		final UserAgent userAgent = UserAgent.parseUserAgentString(ServletUtils.getRequest().getHeader("User-Agent"));
		//获取登录ip
		final String ip = IpUtils.getIpAddr(ServletUtils.getRequest());

		// 获取客户端操作系统
		String os = userAgent.getOperatingSystem().getName();
		// 获取客户端浏览器
		String browser = userAgent.getBrowser().getName();
		// 登录地址
		String address = AddressUtils.getRealAddressByIP(ip);

		userDTO.setLoginTime(new Date());

		userDTO.setLoginIp(ip);
		userDTO.setLoginLocation(address);
		userDTO.setOs(os);
		userDTO.setBrowser(browser);

		return userDTO;
	}

	/**
	 * 根据登录名获取用户
	 * @param name
	 * @return
	 */
	public UserDTO getUserByName(String name) {
		UserDTO userDTO = userMapper.getUserByName(name);

		return userDTO;
	}

	/**
	 * 根据登录名获取用户,不使用缓存
	 * @param loginName
	 * @return
	 */
	public UserDTO getUserByLoginNameNoCache(String loginName) {
		QueryWrapper queryWrapper = new QueryWrapper ();
		queryWrapper.eq ("a.login_name", loginName);
		return baseMapper.get (queryWrapper);
	}

	/**
	 * 根据登录名获取用户id
	 * @param loginName
	 * @return
	 */
	public String getUserIdByLogin(String loginName) {
		return userMapper.getUserIdByLogin (loginName);
	}

	/**
	 * 根据登录名或手机号获取用户信息
	 * @param loginName
	 * @return
	 */
	public List<UserDTO> getUserIdByLoginOrMobile(String loginName) {
		return userMapper.getUserIdByLoginOrMobile (loginName);
	}

	/**
	 * 根据用户name获取用户id
	 * @param name
	 * @return
	 */
	public String getUserIdByName(String name) {
		return userMapper.getUserIdByName (name);
	}


	/**
	 * 自定义分页检索
	 * @param page
	 * @param queryWrapper
	 * @return
	 */
	public IPage <UserDTO> findPage(Page <UserDTO> page, QueryWrapper queryWrapper) {
		queryWrapper.eq ("a.del_flag", 0 ); // 排除已经删除
		return  baseMapper.findList (page, queryWrapper);


	}

	/**
	 * 查询角色下关联的用户
	 * @param page
	 * @param queryWrapper
	 * @return
	 */
	public IPage <UserDTO> findPageByRole(Page <UserDTO> page, QueryWrapper queryWrapper) {
		queryWrapper.eq ("a.del_flag", 0 ); //排除已经删除
		return  baseMapper.findListByRole (page, queryWrapper);


	}

	/**
	 * 查询角色下关联的用户
	 * @param roleId
	 * @return
	 */
	public List <UserDTO> findListByRoleId(String roleId) {
		QueryWrapper queryWrapper = new QueryWrapper (  );
		queryWrapper.eq ("a.del_flag", 0 ); //排除已经删除
		queryWrapper.eq ( "r.id", roleId );

		Page <UserDTO> page = new Page <> (  );
		page.setSize ( -1 );
		return  baseMapper.findListByRole ( page, queryWrapper ).getRecords ();
	}

	/**
	 * 查询岗位下关联的用户
	 * @param postId
	 * @return
	 */
	public List <UserDTO> findListByPostId(String postId) {
		return  baseMapper.findListByPostId ( postId );


	}

	public void saveUserRole(String userId,String roleId){
		if(StrUtil.isNotEmpty(userId)&&StrUtil.isNotEmpty(roleId)){
			baseMapper.insertUserRole ( userId, roleId );
		}
	}

	/**
	 * 保存或者更新用户
	 * @param userDTO
	 */
	public void saveOrUpdate(UserDTO userDTO) throws Exception {
		User user = userWrapper.toEntity (userDTO);
		super.saveOrUpdate ( user );
		if( StrUtil.isBlank ( userDTO.getId () ) ){
			userDTO.setId ( user.getId () );
		}
		// 更新用户与角色关联
		baseMapper.deleteUserRole(userDTO.getId ());
		if (userDTO.getRoleDTOList () != null){
			userDTO.getRoleDTOList ().forEach ( roleDTO -> {
				baseMapper.insertUserRole ( userDTO.getId (), roleDTO.getId () );
			} );
		}else{
			throw new RuntimeException (userDTO.getLoginName() + "没有设置角色！");
		}
		// 更新用户与岗位联系
		baseMapper.deleteUserPost ( userDTO.getId () );
		if(userDTO.getPostDTOList () != null) {
			userDTO.getPostDTOList ().forEach ( postDTO -> {
				baseMapper.insertUserPost ( userDTO.getId (), postDTO.getId () );
			} );
		}
		// 更新用户的管理部门
		List<String> officeIdList = new ArrayList<>();
		if(StringUtils.isNotBlank(userDTO.getManageOfficeIds())){
			List<String> officeIds = Arrays.asList(userDTO.getManageOfficeIds().split(","));
			officeIdList.addAll(officeIds);
		}
		this.updateUserManageOffice(officeIdList, userDTO);

		// 新增/修改关联花名册数据
		this.saveOrUpdateRoster(user);

		UserUtils.deleteCache ( userDTO );
	}


	public void deleteUser(UserDTO userDTO) {
		baseMapper.deleteUserRole(userDTO.getId ());
		baseMapper.deleteUserPost ( userDTO.getId () );
		super.removeById (userDTO.getId ());
		if (StringUtils.isNotBlank(userDTO.getRosterId())) {
			// 删除与用户关联的花名册数据
			rosterBaseCService.removeById(userDTO.getRosterId());
		}
		UserUtils.deleteCache ( userDTO );
	}

	public void updateUserLoginInfo(UserDTO userDTO) {
		User user = new User ( userDTO.getId () );
		// 更新本次登录信息
		HttpServletRequest request = RequestUtils.getRequest ();
		if ( request != null ) {
			user.setLoginIp( request.getRemoteHost () );
		}

		user.setLoginDate(new Date());
		super.updateById (user);
	}

	//查询所有人员信息
	public List<OfficeDTO> disposeUserOffice(List<OfficeDTO> officeDTOList, String type){
		return userMapper.findListByCompanyId(UserUtils.getCurrentUserDTO().getCompanyDTO().getId(),type);
	}

	//根据部门查询用户信息
	public List<OfficeDTO> getUserByOffice(List<String> officeIds,String type){
		return userMapper.getUserByOffice(officeIds,type);
	}

	/**
	 * 根据部门id查询部门下所有有效用户信息
	 * @param officeIds
	 * @return
	 */
	public List<UserDTO> getUserByOfficeIds(List<String> officeIds){
		return userMapper.getUserByOfficeIds(officeIds);
	}


	public LoginUserOnlineDTO selectOnlineByInfo(String ip, String userName, UserDTO user){
		if (StringUtils.equals(ip, user.getIp()) && StringUtils.equals(userName, user.getName())) {
			return loginUserToUserOnline(user);
		}
		return null;
	}

	public LoginUserOnlineDTO selectOnlineByIp(String ip,  UserDTO user){
		if (StringUtils.equals(ip, user.getIp())) {
			return loginUserToUserOnline(user);
		}
		return null;
	}

	public LoginUserOnlineDTO selectOnlineByName(String userName, UserDTO user){
		if (StringUtils.equals(userName, user.getName())) {
			return loginUserToUserOnline(user);
		}
		return null;
	}

	/**
	 * 设置在线用户信息
	 *
	 * @param user 用户信息
	 * @return 在线用户
	 */
	public LoginUserOnlineDTO loginUserToUserOnline(UserDTO user)
	{
		if (null == user ) {
			return null;
		}
		LoginUserOnlineDTO sysUserOnline = new LoginUserOnlineDTO();
		sysUserOnline.setTokenId(user.getToken());
		sysUserOnline.setUserName(user.getName());
		sysUserOnline.setIp(user.getIp());
		sysUserOnline.setLoginLocation(user.getLoginLocation());
		sysUserOnline.setBrowser(user.getBrowser());
		sysUserOnline.setOs(user.getOs());
		sysUserOnline.setLoginTime(user.getLoginTime());
		sysUserOnline.setLoginName(user.getLoginName());
		if(null != user.getOfficeDTO() && StringUtils.isNotBlank(user.getOfficeDTO().getName())){
			sysUserOnline.setOfficeName(user.getOfficeDTO().getName());
		}
		return sysUserOnline;
	}

	/**
	 * 根据部门id获取 部门 以及 子部门 下的所有用户信息
	 * @param officeId
	 * @return
	 */
	public List<User> getUserByOfficeAll(String officeId){
		List<Office> officeList = officeMapper.selectList(new QueryWrapper<Office>().lambda().eq(Office::getParentId, officeId));
		List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
		while(true) {
			List<String> ids = new ArrayList<>();
			officeIdList.stream().forEach(item->{
				List<Office> officeList1 = officeMapper.selectList(new QueryWrapper<Office>().lambda().eq(Office::getParentId, item));
				officeList1.stream().forEach(i->{
					officeList.add(i);
					ids.add(i.getId());
				});
			});
			if (CollectionUtil.isNotEmpty(ids)) {
				officeIdList = ids;
			} else {
				break;
			}
		}
		List<String> officeIds = officeList.stream().distinct().filter(item -> {
			if (StringUtils.isNotBlank(item.getId())) {
				return true;
			}
			return false;
		}).map(Office::getId).collect(Collectors.toList());
		officeIds.add(officeId);
		return userMapper.selectList(new QueryWrapper<User>().lambda().in(User::getOfficeId,officeIds));
	}


	/**
	 * 根据用户手机号和当前人id查询非当前登录人的信息
	 * @param userDTO
	 */
	public List<UserDTO> getUserInfoByMobile(UserDTO userDTO) {
		return userMapper.getUserInfoByMobile(userDTO);
	}

//	保存人员执业资质
	public void saveCertList(UserDTO userDTO) {
		if (ObjectUtil.isNotEmpty(userDTO)) {
			if (CollectionUtil.isNotEmpty(userDTO.getCertDTOList())) {
				List<String> ids = userDTO.getCertDTOList().stream().filter(item -> {
					if (com.jeeplus.sys.utils.StringUtils.isNotBlank(item.getId())) {
						return true;
					}
					return false;
				}).map(CertDTO::getId).collect(Collectors.toList());
				if (CollectionUtil.isNotEmpty(ids)) {
					certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,userDTO.getId()).notIn(Cert::getId,ids));
				}else{
					certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,userDTO.getId()));
				}
				userDTO.getCertDTOList().stream().forEach(item->{
					Cert cert = CertWrapper.INSTANCE.toEntity(item);
					cert.setUserId(userDTO.getId());
					certService.saveOrUpdate(cert);
				});
			} else {
				certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,userDTO.getId()));
			}
		}
	}


	/**
	 * 根据角色名称查询角色下人员信息
	 * @param roleName 角色名称
	 * @return
	 */
	public List<UserDTO> getUserInfoByRoleName(String roleName){
		return userMapper.getUserInfoByRoleName(roleName);
	}

	/**
	 * 根据角色英文名称查询角色下人员信息
	 * @param enName 角色英文名称
	 * @return
	 */
	public List<UserDTO> getUserInfoByEnName(String enName){
		return userMapper.getUserInfoByEnName(enName);
	}

	public Boolean isUpdatePassword(String userId) {
		String updatePassword = userMapper.isUpdatePassword(userId);
		if (StringUtils.isNotBlank(updatePassword)) {
			if ("1".equals(updatePassword)) {
				// 密码已经修改过
				return true;
			}
		}
		return false;
	}


	/**
	 * 修改用户修改密码状态信息
	 * @param userDTO
	 */
	public void updateUserUpPassword(UserDTO userDTO) {
		userMapper.updateUserUpPassword (userDTO);
	}

	public void updateUserManageOffice(List<String> officeIdList, UserDTO userDTO) {
		//根据用户id删除原有用户管理的部门信息
		userMapper.deleteUserManageOffice(userDTO);
		//遍历将新的用户管理的部门信息信息添加进去
		if(officeIdList.size()>0){
			for (String officeId : officeIdList) {
				userMapper.insertUserManageOffice(officeId,userDTO.getId());
			}
		}
	}

	// 新增/修改花名册数据
	public void saveOrUpdateRoster (User user) throws Exception {
		RosterBaseC r = new RosterBaseC();
		r.setName(user.getName());  // 姓名
		r.setOfficeId("");  // 部门
		r.setMobile("");  // 电话号码
		r.setEmail("");  // 邮箱
		if (StringUtils.isNotBlank(user.getOfficeId())) {
			r.setOfficeId(user.getOfficeId());  // 部门
		}
		if (StringUtils.isNotBlank(user.getMobile())) {
			r.setMobile(user.getMobile());  // 电话号码
		}
		if (StringUtils.isNotBlank(user.getEmail())) {
			r.setEmail(user.getEmail());  // 邮箱
		}
		User u = this.getById(user.getId());
		if (StringUtils.isNotBlank(u.getRosterId())) {
			r.setId(u.getRosterId()); // 花名册id
			r.setWorkerType("1");  //  员工类型： 全职
			RosterBaseC roster = rosterBaseCService.getById(u.getRosterId());
			if (ObjectUtil.isNotEmpty(roster)) {
				if (StringUtils.isNotBlank(roster.getWorkerType())) {
					r.setWorkerType(roster.getWorkerType());  //  如果花名册的员工类型有值，则不修改花名册的员工类型
				}
			}
		} else {
			UserDTO userDTO = UserUtils.getCurrentUserDTO();
			String serialNum = serialnumTplCService.genSerialNum(userDTO.getCompanyDTO().getId(), "3");
			r.setJobNo(serialNum); // 花名册工号
			user.setNo(serialNum); // 用户工号
			r.setPersonType("1");  //  人员类型： 员工
			r.setWorkerType("1");  //  员工类型： 全职
		}
		// 保存花名册数据
		rosterBaseCService.saveOrUpdate(r);
		user.setRosterId(r.getId());
		this.updateById(user);
	}

	// 修改用户信息
	public ResponseEntity updateInfo(UserDTO userDTO) throws Exception {
		User user = UserWrapper.INSTANCE.toEntity(userDTO);
		// 根据用户id修改用户信息
		this.updateById (user);
		// 修改用户信息的同时更新花名册数据
		this.saveOrUpdateRoster(user);
		//更新缓存信息
		UserUtils.deleteCache ( UserUtils.getCurrentUserDTO ( ) );
		return ResponseEntity.ok ( "修改个人资料成功!" );
	}

	// 根据用户id查询资格证
	public List<Cert> getCertListByUserId(String userId) {
		return certService.list(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,userId));
	}

	// 根据岗位id查询用户
	public List<User> getUserIdByPost(String postId) {
		return userMapper.getUserByPost(postId);
	}
}
